#!/usr/bin/python

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# Copyright 2001 by Lennart Poettering <mz7862656c6d61726b@poettering.de>
# This script will create a HTML page of your Galeon/XBEL XML Bookmarks

# Further information, usage examples and current versions may be found on
# http://www.stud.uni-hamburg.de/users/lennart/projects/xbelmark/

# 2004-11-22: Modified by Nate Aune <natea@jazkarta.com> to parse the desc 
# of the bookmarks as well
#
# You might want to change the first line to adapt it to your local
# python installation

#2005-02-11 Christian Ledermann
# cleaned up code to use it with plone

import sys, string, re, time, locale, getopt, random
from xml.sax import make_parser, handler, SAXException
from xml.sax.saxutils import *




class Bookmark:
    """ bookmarks will be added as links in plone"""
    def __init__(self, id="", name=u"", url="", desc=u""):
        self.id = id
        self.name = name
        self.url = url
        self.desc = desc


    def __cmp__(self, other):
        return cmp(self.name, other.name)

class Folder:
    """A folder contains bookmarks and folders 
    folder will be added as a BookmarkFolder in plone"""
    def __init__(self, id="", name =u"" , desc=u""):
        self.name = name
        self.id = id
        self.desc = desc
        self.bookmarks = []
        self.subfolders = []

    def addBookmark(self, id ="", name = u"", url = "", desc = u""):
        """ Add bookmark id and url are attributes in xbel
        title and desc are assigned by characters() and endElement() """
        b = Bookmark(id, name, url, desc)
        self.bookmarks.append(b)
        return b

    def addSubfolder(self, id="", name = u"", desc = u""):
        """ id is an attribute in xml
        title and desc are assigned by characters() and endElement() """
        f = Folder(id, name, desc)
        self.subfolders.append(f)
        return f

class ImportHandler(handler.ContentHandler):
    """ Build folders and bookmarks from xml """
    def __init__(self):
        self.title = u""
        self.desc = u""
        self.folderStack = []
        self.currentFolder = None
        self.currentBookmark = None


    def mkBookmarkId(self, anchor):
        """ generate a Bookmarkid from uri """
        bmId = 'lnk_'
        try:
            splitAnchor = string.split(anchor,'//')
            tmpId=splitAnchor [1] [:40]
            for x in tmpId:
                if (('A' <= x <='Z') or ('a' <= x <= 'z') or ('0' <= x <='9')):
                    bmId = bmId + x
                else:
                   bmId= bmId + '-'
        except:
           bmId = 'lnk_' + str(random.randint(1000,99999))
        return bmId
        
    def startDocument(self):
        self.currentFolder = Folder("root")

    def endDocument(self):
        pass

    def startElement(self, name, attrs):
        """assign values to bookmarks and folders """
        if name == "folder":
            self.folderStack.append(self.currentFolder)
            if attrs.has_key("id"):
                bid = attrs["id"]
            else:
                bid = 'bmf_' + str(random.randint(1000,99999))
            self.currentFolder = self.currentFolder.addSubfolder(id = bid, name=u"name", desc=u"no description")

        elif name == "bookmark" and attrs.has_key("href"):
            if attrs.has_key("id"):
                bid = attrs["id"]
            else:
                bid = self.mkBookmarkId(attrs["href"])
            self.currentBookmark = self.currentFolder.addBookmark(id=bid, url=attrs["href"], desc=u"no description")
            
        elif name == "title":
            self.title = u""
        
        elif name == "desc":
            self.desc = u""  

    def endElement(self, name):
        """ clean up folders and bookmarks, set desc and title """
        if name == "folder":
            self.currentFolder = self.folderStack.pop()

        elif name == "bookmark":
            self.currentBookmark = None

        elif name == "title":
            if not self.currentBookmark is None:
                self.currentBookmark.name = self.correctCharset(self.title)
            elif not self.currentFolder is None:
                self.currentFolder.name = self.correctCharset(self.title)

        elif name == "desc":
            if not self.currentBookmark is None:
                self.currentBookmark.desc = self.correctCharset(self.desc)
            elif not self.currentFolder is None:
                self.currentFolder.desc = self.correctCharset(self.desc)
            
    def characters(self, content):
            """ assign title and desc """
            try:
                self.title += content
                self.desc += content
            except:
                pass
            
    def correctCharset(self, s):
        """ not yet imlemented """
        return (s) #unicode(s.encode("iso8859-1"), "utf-8")

    
def main():
    """ does nothing """
    pass

if __name__ == "__main__":
    main()
